# Generated by roxygen2: do not edit by hand

export(AUC_lnK_R_NX)
export(DRR)
export(DiffusionMaps)
export(DrL)
export(FastICA)
export(FruchtermanReingold)
export(HLLE)
export(Isomap)
export(KamadaKawai)
export(LCMC)
export(MDS)
export(NNMF)
export(PCA)
export(PCA_L1)
export(Q_NX)
export(Q_global)
export(Q_local)
export(R_NX)
export(UMAP)
export(dataSetList)
export(dimRedData)
export(dimRedMethodList)
export(dimRedQualityList)
export(dimRedResult)
export(distance_correlation)
export(embed)
export(getRotationMatrix)
export(installSuggests)
export(inverse)
export(kPCA)
export(loadDataSet)
export(mean_R_NX)
export(mixColor1Ramps)
export(mixColor2Ramps)
export(mixColor3Ramps)
export(mixColorRamps)
export(nMDS)
export(plot)
export(plot_R_NX)
export(predict)
export(quality)
export(reconstruction_error)
export(reconstruction_rmse)
export(tSNE)
export(total_correlation)
exportClasses(DRR)
exportClasses(DiffusionMaps)
exportClasses(DrL)
exportClasses(FastICA)
exportClasses(FruchtermanReingold)
exportClasses(HLLE)
exportClasses(Isomap)
exportClasses(KamadaKawai)
exportClasses(MDS)
exportClasses(NNMF)
exportClasses(PCA)
exportClasses(PCA_L1)
exportClasses(UMAP)
exportClasses(dimRedData)
exportClasses(dimRedMethod)
exportClasses(dimRedResult)
exportClasses(kPCA)
exportClasses(nMDS)
exportClasses(tSNE)
exportMethods("[")
exportMethods(AUC_lnK_R_NX)
exportMethods(LCMC)
exportMethods(Q_NX)
exportMethods(Q_global)
exportMethods(Q_local)
exportMethods(R_NX)
exportMethods(as.data.frame)
exportMethods(as.dimRedData)
exportMethods(cophenetic_correlation)
exportMethods(distance_correlation)
exportMethods(embed)
exportMethods(getData)
exportMethods(getDimRedData)
exportMethods(getMeta)
exportMethods(getNDim)
exportMethods(getOrgData)
exportMethods(getOtherData)
exportMethods(getPars)
exportMethods(inverse)
exportMethods(maximize_correlation)
exportMethods(mean_R_NX)
exportMethods(ndims)
exportMethods(nrow)
exportMethods(plot)
exportMethods(predict)
exportMethods(print)
exportMethods(quality)
exportMethods(reconstruction_error)
exportMethods(reconstruction_rmse)
exportMethods(total_correlation)
import(DRR)
import(methods)
import(utils)
importFrom(grDevices,colorRamp)
importFrom(grDevices,rgb)
importFrom(graphics,plot)
importFrom(magrittr,"%>%")
importFrom(stats,predict)
