% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{getData}
\alias{getData}
\title{Method getData}
\usage{
getData(object)
}
\arguments{
\item{object}{The object to be converted.}
}
\description{
Extracts the data slot.
}
