% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quality.R
\name{distance_correlation,dimRedResult-method}
\alias{distance_correlation,dimRedResult-method}
\alias{distance_correlation}
\title{Method distance_correlation}
\usage{
\S4method{distance_correlation}{dimRedResult}(object)
}
\arguments{
\item{object}{of class dimRedResult}
}
\description{
Calculate the distance correlation between the distance matrices in
high and low dimensioal space.
}
\seealso{
Other Quality scores for dimensionality reduction: 
\code{\link{AUC_lnK_R_NX,dimRedResult-method}},
\code{\link{LCMC,dimRedResult-method}},
\code{\link{Q_NX,dimRedResult-method}},
\code{\link{Q_global,dimRedResult-method}},
\code{\link{Q_local,dimRedResult-method}},
\code{\link{R_NX,dimRedResult-method}},
\code{\link{cophenetic_correlation,dimRedResult-method}},
\code{\link{mean_R_NX,dimRedResult-method}},
\code{\link{plot_R_NX}()},
\code{\link{quality,dimRedResult-method}},
\code{\link{reconstruction_error,dimRedResult-method}},
\code{\link{reconstruction_rmse,dimRedResult-method}},
\code{\link{total_correlation,dimRedResult-method}}
}
\concept{Quality scores for dimensionality reduction}
