% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{getPars}
\alias{getPars}
\title{Method getPars}
\usage{
getPars(object, ...)
}
\arguments{
\item{object}{The object to be converted.}

\item{...}{other arguments.}
}
\description{
Extracts the pars slot.
}
