% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/misc.R
\name{getMeta}
\alias{getMeta}
\title{Method getMeta}
\usage{
getMeta(object, ...)
}
\arguments{
\item{object}{The object to be converted.}

\item{...}{other arguments.}
}
\description{
Extracts the meta slot.
}
