% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quality.R
\name{Q_local,dimRedResult-method}
\alias{Q_local,dimRedResult-method}
\alias{Q_local}
\title{Method Q_local}
\usage{
\S4method{Q_local}{dimRedResult}(object, ndim = getNDim(object))
}
\arguments{
\item{object}{of class dimRedResult.}

\item{ndim}{use the first ndim columns of the embedded data for calculation.}
}
\description{
Calculate the Q_local score to assess the quality of a dimensionality reduction.
}
\seealso{
Other Quality scores for dimensionality reduction: 
\code{\link{AUC_lnK_R_NX,dimRedResult-method}},
\code{\link{LCMC,dimRedResult-method}},
\code{\link{Q_NX,dimRedResult-method}},
\code{\link{Q_global,dimRedResult-method}},
\code{\link{R_NX,dimRedResult-method}},
\code{\link{cophenetic_correlation,dimRedResult-method}},
\code{\link{distance_correlation,dimRedResult-method}},
\code{\link{mean_R_NX,dimRedResult-method}},
\code{\link{plot_R_NX}()},
\code{\link{quality,dimRedResult-method}},
\code{\link{reconstruction_error,dimRedResult-method}},
\code{\link{reconstruction_rmse,dimRedResult-method}},
\code{\link{total_correlation,dimRedResult-method}}
}
\concept{Quality scores for dimensionality reduction}
